﻿using Microsoft.AspNetCore.Mvc.ApplicationParts;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Falcon.SugarApi.Plugin.Service
{
    /// <summary>
    /// 服务扩展 服务端调用
    /// </summary>
    public static class IServiceCollectionExtend
    {
        /// <summary>
        /// 注册插件。
        /// <para>查找插件中的IServicePlugin实现，并运行其中的AddServices方法注册和初始化插件</para>
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="config">应用程序配置</param>
        /// <param name="pluginNames">插件名称数组</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginService(this IServiceCollection services,IConfiguration config,params string[] pluginNames) {
            var name = typeof(IServicePlugin).FullName;
            if(name == null) {
                return services;
            }
            foreach(var pluginName in pluginNames) {
                var pa = AssemblyLoadHelp.ALCLoad(pluginName);
                foreach(Type type in pa.GetTypes()) {
                    if(type == null || !type.IsPublic || type.GetInterface(name) == null) {
                        continue;
                    }
                    var tname = type.FullName;
                    if(tname == null) {
                        continue;
                    }
                    if(type.Assembly.CreateInstance(tname) is not IServicePlugin obj) {
                        continue;
                    }
                    obj.AddServices(services,config);
                }

            }
            return services;
        }

        /// <summary>
        /// 注册插件中的控制器。一般由插件自行注册。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="assemblyPath">插件程序集文件路径</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsController(this IServiceCollection services,string assemblyPath) {
            var pluginAssembly = Assembly.LoadFrom(assemblyPath);
            return services.AddPluginsController(pluginAssembly);
        }

        /// <summary>
        /// 注册插件中的控制器。一般由插件自行注册。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="assembly">插件程序集</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddPluginsController(this IServiceCollection services,Assembly assembly) {
            var part = new AssemblyPart(assembly);
            services.AddControllers().ConfigureApplicationPartManager(apm => {
                if(!apm.ApplicationParts.Contains(part)) {
                    apm.ApplicationParts.Add(part);
                }
            });
            return services;
        }

    }
}
